package cn.zhangdeman.exception.handler;

import cn.zhangdeman.consts.RecordField;
import cn.zhangdeman.context.Response;
import cn.zhangdeman.context.RuntimeContext;
import cn.zhangdeman.exception.CustomException;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.context.request.WebRequest;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestControllerAdvice // 适用于REST API，返回JSON响应
public class GlobalExceptionHandler {
    // 获取runtime context
    private RuntimeContext getRuntimeContext(HttpServletRequest httpServletRequest) {
        Object object = httpServletRequest.getAttribute(RecordField.RUNTIME_THREAD_CONTEXT);
        return (RuntimeContext) object;
    }

    // 构建相应实体
    private ResponseEntity<Object> getResponseEntity(Response<Object> response) {
        ResponseEntity.BodyBuilder builder = ResponseEntity.status(response.getHttpCode());
        for (Map.Entry<String, String> header : response.getHeader().entrySet()) {
            builder.header(header.getKey(), header.getValue());
        }
        return ResponseEntity.status(response.getHttpCode())
                .body(response);
    }

    // 处理自定义业务异常
    @ExceptionHandler(cn.zhangdeman.exception.CustomException.class)
    public ResponseEntity<Object> handleBusinessException(CustomException customException, HttpServletRequest httpServletRequest) {
        RuntimeContext runtimeContext = getRuntimeContext(httpServletRequest);
        Response<Object> response = new Response<>(runtimeContext).failure(customException);
        return getResponseEntity(response);
    }

    // 处理参数校验异常（如@NotNull、@Size等校验失败）对应validate库
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseEntity<Object> handleValidationException(MethodArgumentNotValidException ex, HttpServletRequest httpServletRequest) {
        List<Map<String, Object>> errDetailList = new ArrayList<>();
        List<String> errors = new ArrayList<>();
        ex.getBindingResult().getFieldErrors().forEach(itemErr -> {
            // message 层面显示的信息
            errors.add(itemErr.getField() + ":" + itemErr.getDefaultMessage());
            // err detail信息
            Map<String, Object> detail = new HashMap<>();
            detail.put("object_name", itemErr.getObjectName());
            detail.put("field", itemErr.getField());
            detail.put("input", itemErr.getRejectedValue());
            detail.put("code", itemErr.getCode());
            detail.put("binding_failure", itemErr.isBindingFailure());
            errDetailList.add(detail);
        });

        RuntimeContext runtimeContext = getRuntimeContext(httpServletRequest);
        Response<Object> response = new Response<>(runtimeContext).failure("400", String.join(" - ", errors), errDetailList);
        return getResponseEntity(response);
    }


    // 处理其他未捕获的异常
    @ExceptionHandler(Exception.class)
    public ResponseEntity<Object> handleGlobalException(Exception ex, HttpServletRequest httpServletRequest) {
        RuntimeContext runtimeContext = getRuntimeContext(httpServletRequest);
        Response<Object> response = new Response<>(runtimeContext).failure("500", ex.getMessage(), ex.getCause());
        return getResponseEntity(response);
    }
}
